/*
 * linux/arch/arm/mach-pxa/limestone.c
 *
 * Support for the Toradex Limestone PDA Design Kit V1.2
 *
 * Copyright (C) 2008-2011 Noser Engineering
 *
 * 2008-09-17: Marcel Ziswiler <marcel.ziswiler@noser.com>
 *             initial version
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach-types.h>

#include <linux/gpio_keys.h>
#include <linux/init.h>
#include <linux/input.h>
#include <linux/interrupt.h>
#include <linux/kernel.h>
#include <linux/leds.h>
#include <linux/platform_device.h>
#include <linux/spi/spi.h>
#if defined(CONFIG_USB_ANDROID) || defined(CONFIG_USB_ANDROID_MODULE)
#include <linux/usb/android_composite.h>
#endif
#include <linux/usb/gpio_vbus.h>

#include <mach/audio.h>
#include <mach/gpio.h>
#include <mach/mfp-pxa320.h>
#include <mach/ohci.h>
#include <mach/pxa2xx_spi.h>
#include <mach/pxa27x-udc.h>
#include <mach/pxa3xx-regs.h>
#include <mach/pxafb.h>
#include <mach/udc.h>

#include <plat/i2c.h>
#include <plat/pxa3xx_nand.h>

#include "generic.h"
#include "colibri.h"

#define LIMESTONE_GPIO_USB_OC		mfp_to_gpio(MFP_PIN_GPIO2_2)
#define LIMESTONE_GPIO_USB_PEN		mfp_to_gpio(MFP_PIN_GPIO3_2)
#define LIMESTONE_GPIO_BACKLIGHT	mfp_to_gpio(MFP_PIN_GPIO11)
#define LIMESTONE_GPIO_DISP_ON		mfp_to_gpio(MFP_PIN_GPIO16)
#define LIMESTONE_GPIO_KEY_HOME		mfp_to_gpio(MFP_PIN_GPIO24)
#define LIMESTONE_GPIO_KEY_MENU		mfp_to_gpio(MFP_PIN_GPIO25)
#define LIMESTONE_GPIO_KEY_BACK		mfp_to_gpio(MFP_PIN_GPIO26)
#define LIMESTONE_GPIO_VIBRA		mfp_to_gpio(MFP_PIN_GPIO42)
#define LIMESTONE_GPIO_USB_ID		mfp_to_gpio(MFP_PIN_GPIO96)

static mfp_cfg_t mfp_cfg[] __initdata = {
#if defined(CONFIG_SND_PXA2XX_AC97) || defined(CONFIG_SND_PXA2XX_AC97_MODULE)
	GPIO34_AC97_SYSCLK,			/* MCLKA */
	GPIO35_AC97_SDATA_IN_0 | MFP_LPM_EDGE_BOTH,
	GPIO36_GPIO,				/* Touchscreen interrupt */
	GPIO37_AC97_SDATA_OUT,
	GPIO38_AC97_SYNC,
	GPIO39_AC97_BITCLK | MFP_LPM_EDGE_BOTH,
	GPIO40_AC97_nACRESET,
#endif /* CONFIG_SND_PXA2XX_AC97 || CONFIG_SND_PXA2XX_AC97_MODULE */

#if defined(CONFIG_I2C_PXA) || defined(CONFIG_I2C_PXA_MODULE)
	GPIO32_I2C_SCL | MFP_LPM_EDGE_BOTH,
	GPIO33_I2C_SDA | MFP_LPM_EDGE_BOTH,
#endif /* CONFIG_I2C_PXA || CONFIG_I2C_PXA_MODULE */

#if defined(CONFIG_KEYBOARD_GPIO) || defined(CONFIG_KEYBOARD_GPIO_MODULE)
	/* Keys (borrowed from MMC2)
	   SD socket pin out (top back side)
	   9     1     2     3     4     5     6     7     8     10    11
	   DAT2  DAT3  CMD   GND   3V3   CLK   GND   DAT0  DAT1  CD    WP
	   Back                    Power             Home  Menu */
	GPIO24_GPIO | MFP_LPM_EDGE_RISE, /* Home */
	GPIO25_GPIO | MFP_LPM_EDGE_RISE, /* Menu */
	GPIO26_GPIO | MFP_LPM_EDGE_RISE, /* Back */
#endif /* CONFIG_KEYBOARD_GPIO || CONFIG_KEYBOARD_GPIO_MODULE */

#if defined(CONFIG_FB_PXA) || defined(CONFIG_FB_PXA_MODULE)
	GPIO6_2_LCD_LDD_0,
	GPIO7_2_LCD_LDD_1,
	GPIO8_2_LCD_LDD_2,
	GPIO9_2_LCD_LDD_3,
	GPIO10_2_LCD_LDD_4,
	GPIO11_GPIO,		/* Backlight, high active */
	GPIO11_2_LCD_LDD_5,
	GPIO12_2_LCD_LDD_6,
	GPIO13_2_LCD_LDD_7,
	GPIO16_GPIO,		/* DispOn, high active */
	GPIO63_LCD_LDD_8,
	GPIO64_LCD_LDD_9,
	GPIO65_LCD_LDD_10,
	GPIO66_LCD_LDD_11,
	GPIO67_LCD_LDD_12,
	GPIO68_LCD_LDD_13,
	GPIO69_LCD_LDD_14,
	GPIO70_LCD_LDD_15,
	GPIO71_LCD_LDD_16,
	GPIO72_LCD_LDD_17,
	GPIO14_2_LCD_FCLK,
	GPIO15_2_LCD_LCLK,
	GPIO16_2_LCD_PCLK,
	GPIO17_2_LCD_BIAS,
#endif /* CONFIG_FB_PXA || CONFIG_FB_PXA_MODULE */

#if defined(CONFIG_SPI_PXA2XX) || defined(CONFIG_SPI_PXA2XX_MODULE)
	/* SPI */
	GPIO83_SSP1_SCLK | MFP_LPM_EDGE_BOTH,
	GPIO84_SSP1_SFRM | MFP_LPM_EDGE_BOTH,
	GPIO85_SSP1_RXD | MFP_LPM_EDGE_BOTH,
	GPIO86_SSP1_TXD,
#endif /* CONFIG_SPI_PXA2XX || CONFIG_SPI_PXA2XX_MODULE */

#if defined(CONFIG_SERIAL_PXA) || defined(CONFIG_SERIAL_PXA_MODULE)
	/* FFUART */
	GPIO97_UART1_RXD | MFP_LPM_EDGE_FALL,
	GPIO98_UART1_TXD,
	GPIO99_UART1_CTS | MFP_LPM_EDGE_BOTH,
	GPIO100_UART1_DCD | MFP_LPM_EDGE_BOTH,
	GPIO101_UART1_DSR | MFP_LPM_EDGE_BOTH,
	GPIO102_UART1_RI | MFP_LPM_EDGE_BOTH,
	GPIO103_UART1_DTR,
	GPIO104_UART1_RTS,

	/* BTUART */
	GPIO109_UART2_CTS | MFP_LPM_EDGE_BOTH,
	GPIO110_UART2_RXD | MFP_LPM_EDGE_FALL,
	GPIO111_UART2_TXD,
	GPIO112_UART2_RTS,

	/* STUART */
	GPIO30_UART3_RXD | MFP_LPM_EDGE_FALL,
	GPIO31_UART3_TXD,
#endif /* CONFIG_SERIAL_PXA || CONFIG_SERIAL_PXA_MODULE */

#if defined(CONFIG_USB_OHCI_HCD) || defined(CONFIG_USB_OHCI_HCD_MODULE)
	GPIO2_2_GPIO,	/* USB host OverCurrent, low active */
	GPIO3_2_GPIO,	/* USB host PowerEn, high active */
#endif /* CONFIG_USB_OHCI_HCD || CONFIG_USB_OHCI_HCD_MODULE */

#if defined(CONFIG_USB_GADGET_PXA27X) || \
    defined(CONFIG_USB_GADGET_PXA27X_MODULE)
	GPIO96_GPIO,	/* USB ID, low = host */
#endif /* CONFIG_USB_GADGET_PXA27X || CONFIG_USB_GADGET_PXA27X_MODULE */

#if defined(CONFIG_LEDS_GPIO) || defined(CONFIG_LEDS_GPIO_MODULE)
	/* Vibra motor, low active */
	GPIO42_GPIO,
	GPIO105_GPIO,	/* Multiplexed */
#endif /* CONFIG_LEDS_GPIO || CONFIG_LEDS_GPIO_MODULE */
};

#if defined(CONFIG_SND_PXA2XX_AC97) || defined(CONFIG_SND_PXA2XX_AC97_MODULE)
static void __init limestone_init_ac97(void)
{
	pxa_set_ac97_info(NULL);
}
#else /* CONFIG_SND_PXA2XX_AC97 || CONFIG_SND_PXA2XX_AC97_MODULE */
static inline void limestone_init_ac97(void) {}
#endif /* CONFIG_SND_PXA2XX_AC97 || CONFIG_SND_PXA2XX_AC97_MODULE */

#if defined(CONFIG_I2C_PXA) || defined(CONFIG_I2C_PXA_MODULE)
static void __init limestone_init_i2c(void)
{
	pxa_set_i2c_info(NULL);
}
#else /* CONFIG_I2C_PXA || CONFIG_I2C_PXA_MODULE */
static inline void limestone_init_i2c(void) {}
#endif /* CONFIG_I2C_PXA || CONFIG_I2C_PXA_MODULE */

#if defined(CONFIG_FB_PXA) || defined(CONFIG_FB_PXA_MODULE)
static void limestone_backlight_power(int on)
{
	gpio_set_value(LIMESTONE_GPIO_BACKLIGHT, on);
}

static void limestone_lcd_power(int on, struct fb_var_screeninfo *var)
{
	gpio_set_value(LIMESTONE_GPIO_DISP_ON, on);
}

/* This supports the 3.5" HITACHI TX09D40VM3CBA QVGA touch TFT panel. */
static struct pxafb_mode_info hitachi_tx09d40vm3cba_mode = {
	/* pixel clock period in picoseconds (5.6 MHz) */
	.pixclock	= 178571,
	.xres		= 240,
	.yres		= 320,
	.bpp		= 16,
	.hsync_len	= 2,	/* pixels horizontal sync */
	.left_margin	= 10,	/* pixels left border */
	.right_margin	= 18,	/* pixels right border */
	.vsync_len	= 2,	/* lines vertical sync */
	.upper_margin	= 6,	/* lines top border */
	.lower_margin	= 20,	/* lines bottom border */
	.sync		= 0,
};

static struct pxafb_mach_info hitachi_tx09d40vm3cba_info = {
	.modes			= &hitachi_tx09d40vm3cba_mode,
	.num_modes		= 1,
	.lcd_conn		= LCD_COLOR_TFT_16BPP,
	.pxafb_backlight_power	= limestone_backlight_power,
	.pxafb_lcd_power	= limestone_lcd_power,
};

static void __init limestone_init_fb(void)
{
	/* backlight GPIO: output, default on */
	gpio_request(LIMESTONE_GPIO_BACKLIGHT, "LCD backlight");
	gpio_direction_output(LIMESTONE_GPIO_BACKLIGHT, 1);

	/* disp on GPIO: output, default on */
	gpio_request(LIMESTONE_GPIO_DISP_ON, "LCD display on");
	gpio_direction_output(LIMESTONE_GPIO_DISP_ON, 1);

	set_pxa_fb_info(&hitachi_tx09d40vm3cba_info);
}
#else /* CONFIG_FB_PXA || CONFIG_FB_PXA_MODULE */
static inline void limestone_init_fb(void) {}
#endif /* CONFIG_FB_PXA || CONFIG_FB_PXA_MODULE */

#if defined(CONFIG_MTD_NAND_PXA3xx) || defined(CONFIG_MTD_NAND_PXA3xx_MODULE)
/* Samsung K9K8G08U0A */
static struct pxa3xx_nand_timing samsung8GbX8_timing = {
	.tCH	= 5,		/* tCH, Enable signal hold time */
	.tCS	= 20,		/* tCS, Enable signal setup time */
	.tWH	= 10,		/* tWH, ND_nWE high duration */
	.tWP	= 12,		/* tWP, ND_nWE pulse time */
	.tRH	= 100,		/* tRHZ, ND_nRE high duration */
	.tRP	= 12,		/* tRP, ND_nRE pulse width */
	.tR	= 20000,	/* tR, ND_nWE high to ND_nRE low for read */
	.tWHR	= 60,		/* tWHR, ND_nWE high to ND_nRE low delay for
				   status read */
	.tAR	= 10,		/* tAR, ND_ALE low to ND_nRE low delay */
};

/* large page NAND command codes (2nd cycle, 1st cycle) */
static struct pxa3xx_nand_cmdset largepage_cmdset = {
	.read1		= 0x3000,	/* Read */
	.read2		= 0x0050,	/* Read2 unused, current DFC doesn't
					   support */
	.program	= 0x1080,	/* Write, two cycle command */
	.read_status	= 0x0070,	/* Read status */
	.read_id	= 0x0090,	/* Read ID */
	.erase		= 0xD060,	/* Block erase, two cycle command */
	.reset		= 0x00FF,	/* Reset */
	.lock		= 0x002A,	/* Lock whole flash */
	.unlock		= 0x2423,	/* Unlock block range */
	.lock_status	= 0x007A,	/* Read block lock status */
};

static struct pxa3xx_nand_flash samsung8GbX8 = {
	.timing		= &samsung8GbX8_timing,
	.cmdset		= &largepage_cmdset,
	.page_per_block	= 64,		/* Pages per block */
	.page_size	= 2048,		/* Page size in bytes */
	.flash_width	= 8,		/* Width of Flash memory */
	.dfc_width	= 8,		/* Width of flash controller */
	.num_blocks	= 8192,		/* Number of physical blocks in
					   Flash */
	.chip_id	= 0xd3ec,	/* chip ID, vendor ID */
};

static struct mtd_partition limestone_pxa320_nand_partitions[] = {
	{
		name:		"Boot loader",
		offset:		0x00000000,
		size:		0x00060000,
		mask_flags:	MTD_WRITEABLE,		/* force read-only */
	}, {
		name:		"Boot loader environment",
		offset:		MTDPART_OFS_APPEND,	/* append after boot
							   loader section */
		size:		0x000A0000,
		mask_flags:	MTD_WRITEABLE,		/* force read-only */
	}, {
		name:		"Kernel",
		offset:		MTDPART_OFS_APPEND,	/* append after boot
							   loader environment
							   area */
		size:		0x00300000,		/* 3 MB */
	}, {
		name:		"system",
		offset:		MTDPART_OFS_APPEND,	/* append after kernel
							   section */
		size:		SZ_256M,		/* 256 MB */
	}, {
		name:		"userdata",
		offset:		MTDPART_OFS_APPEND,	/* append after system
							   section */
		size:		SZ_512M,		/* 512 MB */
	}, {
		name:		"cache",
		offset:		MTDPART_OFS_APPEND,	/* append after
							   userdata section */
		size:		SZ_128M,		/* 128 MB */
	}, {
		name:		"Replacement blocks",
		offset:		MTDPART_OFS_APPEND,	/* append after cache
							   section */
		size:		MTDPART_SIZ_FULL,
		mask_flags:	MTD_WRITEABLE,		/* force read-only */
	}
};

struct pxa3xx_nand_platform_data limestone_nand_info = {
	.enable_arbiter	= 1,
	.keep_config	= 0,
	.parts		= limestone_pxa320_nand_partitions,
	.nr_parts	= ARRAY_SIZE(limestone_pxa320_nand_partitions),
	.flash		= &samsung8GbX8,
	.num_flash	= 1,
};

static void __init limestone_init_nand(void)
{
	pxa3xx_set_nand_info(&limestone_nand_info);
}
#else
static inline void limestone_init_nand(void) {}
#endif /* CONFIG_MTD_NAND_PXA3xx || CONFIG_MTD_NAND_PXA3xx_MODULE */

#if defined(CONFIG_BATTERY_BQ27x00) || defined(CONFIG_BATTERY_BQ27x00_MODULE)
static struct i2c_board_info __initdata limestone_i2c_power = {
	.type	= "bq27200",
	.addr	= 0x55,
};

static void __init limestone_init_power(void)
{
	i2c_register_board_info(0, &limestone_i2c_power, 1);
}
#else /* CONFIG_BATTERY_BQ27x00 || CONFIG_BATTERY_BQ27x00_MODULE */
static inline void limestone_init_power(void) {}
#endif /* CONFIG_BATTERY_BQ27x00 || CONFIG_BATTERY_BQ27x00_MODULE */

#if defined(CONFIG_SPI_PXA2XX) || defined(CONFIG_SPI_PXA2XX_MODULE)
static struct pxa2xx_spi_master pxa_ssp_master_info = {
	.num_chipselect	= 1, /* Matches the number of chips attached to SSP */
	.enable_dma	= 0, /* Enables SSP DMA */
};

static struct platform_device pxa_ssp = {
	.name		= "pxa2xx-spi", /* MUST BE THIS VALUE, so device match
					   driver */
	.id		= 1,  /* Bus number, MUST MATCH SSP number 1..n */
	.dev = {
		.platform_data	= &pxa_ssp_master_info, /* Passed to driver */
	},
};

#if defined(CONFIG_SPI_SPIDEV) || defined(CONFIG_SPI_SPIDEV_MODULE)
static struct pxa2xx_spi_chip spidev_chip_info = {
	.tx_threshold		= 4, /* SSP hardward FIFO threshold */
	.rx_threshold		= 12, /* SSP hardward FIFO threshold */
	.dma_burst_size		= 8,
	.timeout		= 235,
	.enable_loopback	= 1,
};

static struct spi_board_info __initdata limestone_spi_board_info[] = {
	{
		.modalias		= "spidev",
		.max_speed_hz		= 13000000, /* max SPI clock (SCK)
						      speed in HZ */
		.bus_num		= 1,
		.chip_select		= 0,
		.controller_data	= &spidev_chip_info,
		.mode			= SPI_MODE_0,
	},
};
#endif /* CONFIG_SPI_SPIDEV || CONFIG_SPI_SPIDEV_MODULE */

static void __init limestone_init_spi(void)
{
	platform_device_register(&pxa_ssp);
#if defined(CONFIG_SPI_SPIDEV) || defined(CONFIG_SPI_SPIDEV_MODULE)
	spi_register_board_info(limestone_spi_board_info,
				ARRAY_SIZE(limestone_spi_board_info));
#endif /* CONFIG_SPI_SPIDEV || CONFIG_SPI_SPIDEV_MODULE */
}
#else /* CONFIG_SPI_PXA2XX || CONFIG_SPI_PXA2XX_MODULE */
static inline void limestone_init_spi(void) {}
#endif /* CONFIG_SPI_PXA2XX || CONFIG_SPI_PXA2XX_MODULE */

#if defined(CONFIG_SERIAL_PXA) || defined(CONFIG_SERIAL_PXA_MODULE)
static void __init limestone_init_uart(void)
{
	pxa_set_ffuart_info(NULL);
	pxa_set_btuart_info(NULL);
	pxa_set_stuart_info(NULL);
}
#else /* CONFIG_SERIAL_PXA || CONFIG_SERIAL_PXA_MODULE */
static inline void limestone_init_uart(void) {}
#endif /* CONFIG_SERIAL_PXA || CONFIG_SERIAL_PXA_MODULE */

#if defined(CONFIG_USB_ANDROID) || defined(CONFIG_USB_ANDROID_MODULE)
#define GOOGLE_VENDOR_ID	0x18d1
#define GOOGLE_PRODUCT_ID	0x9018

#ifdef CONFIG_USB_ANDROID_ADB
static char *limestone_android_usb_functions_adb[] = {
	"adb",
};
#endif

static char *limestone_android_usb_functions_all[] = {
#ifdef CONFIG_USB_ANDROID_ADB
	"adb",
#endif
};

static struct android_usb_product limestone_android_usb_products[] = {
	[0] = {
		.product_id = GOOGLE_PRODUCT_ID,
		.num_functions = ARRAY_SIZE(limestone_android_usb_functions_adb),
		.functions = limestone_android_usb_functions_adb,
	},
};

static struct android_usb_platform_data limestone_android_usb_platform = {
	.functions = limestone_android_usb_functions_all,
	.manufacturer_name = "Toradex",
	.num_functions = ARRAY_SIZE(limestone_android_usb_functions_all),
	.num_products = ARRAY_SIZE(limestone_android_usb_products),
	.product_id = GOOGLE_PRODUCT_ID,
	.products = limestone_android_usb_products,
	.vendor_id = GOOGLE_VENDOR_ID,
};

static struct platform_device limestone_android_usb_device = {
	.name = "android_usb",
	.id = -1,
	.dev = {
		.platform_data = &limestone_android_usb_platform,
	},
};

static struct platform_device *limestone_platform_devices[] = {
#ifdef CONFIG_USB_ANDROID
	&limestone_android_usb_device,
#endif
};

static void __init limestone_init_usb_android(void)
{
	char serial[17];
	snprintf(serial, sizeof(serial), "%08x%08x", system_serial_high, system_serial_low);
	limestone_android_usb_platform.serial_number = kstrdup(serial, GFP_KERNEL);

	platform_add_devices(limestone_platform_devices, ARRAY_SIZE(limestone_platform_devices));
}
#else
static void inline limestone_init_usb_android(void) {}
#endif /* CONFIG_USB_ANDROID || CONFIG_USB_ANDROID_MODULE */

#if defined(CONFIG_USB_GADGET_PXA27X) || \
    defined(CONFIG_USB_GADGET_PXA27X_MODULE)
static struct pxa2xx_udc_mach_info limestone_udc_info __initdata = {
	/* no D+ pullup; Limestone can't connect/disconnect in software */
	.gpio_pullup = -1,
	/* gpio_vbus only used in pxa25x_udc driver */
};

static struct gpio_vbus_mach_info limestone_gpio_vbus_info = {
	/* no D+ pullup; Limestone can't connect/disconnect in software */
	.gpio_pullup	= -1,
	.gpio_vbus	= LIMESTONE_GPIO_USB_ID,
};

static struct platform_device limestone_gpio_vbus = {
	.name	= "gpio-vbus",
	.id	= -1,
	.dev	= {
		.platform_data	= &limestone_gpio_vbus_info,
	},
};

static void __init limestone_init_udc(void)
{
	pxa_set_udc_info(&limestone_udc_info);
	platform_device_register(&limestone_gpio_vbus);
}
#else
static inline void limestone_init_udc(void) {}
#endif /* CONFIG_USB_GADGET_PXA27X || CONFIG_USB_GADGET_PXA27X_MODULE */

#if defined(CONFIG_USB_OHCI_HCD) || defined(CONFIG_USB_OHCI_HCD_MODULE)
static irqreturn_t limestone_over_current_int(int irq, void *ptr)
{
	/* Check OverCurrent GPIO2_2 (Low active) */
	if (gpio_get_value(irq_to_gpio(irq))) {
		/* Turn power on again */
		gpio_set_value(LIMESTONE_GPIO_USB_PEN, 1);
		printk(KERN_INFO "%s: USB: host power restored.\n", __func__);
	} else {
		/* Shut power off immediately */
		gpio_set_value(LIMESTONE_GPIO_USB_PEN, 0);
		printk(KERN_ERR "%s: USB: host over current!\n", __func__);
	}

	return IRQ_HANDLED;
}

static int limestone_ohci_init(struct device *dev)
{
	int err = 0;
	int gpio_hpe, gpio_ocd, ocd_irq;

	/* Set port 2 to host and enable D +/- pull-down, by default. */
	UP2OCR = UP2OCR_HXOE | UP2OCR_HXS;	/* use port 2 as host */
	UP2OCR |= UP2OCR_DPPDE | UP2OCR_DMPDE;	/* enable D +/- pull-down */

	/*
	 * setup USB host over current detect interrupt
	 */
	gpio_ocd = LIMESTONE_GPIO_USB_OC;
	err = gpio_request(gpio_ocd, "USB host over current detect");
	if (err)
		goto err_request_ocd;
	gpio_direction_input(gpio_ocd);

	ocd_irq = gpio_to_irq(gpio_ocd);
	err = request_irq(ocd_irq, limestone_over_current_int,
			  IRQF_TRIGGER_RISING | IRQF_TRIGGER_FALLING,
			  "USB host over current detect", NULL);
	if (err) {
		printk(KERN_ERR "%s: USB: "
				"can't request host over current detect IRQ\n",
				__func__);
		goto err_request_oc_irq;
	}

	/*
	 * setup and activate USB host power enable (PowerEn) GPIO3_2
	 * (High active)
	 */
	gpio_hpe = LIMESTONE_GPIO_USB_PEN;
	err = gpio_request(gpio_hpe, "USB host power enable");
	if (err)
		goto err_request_hpe;
	gpio_direction_output(gpio_hpe, 0);
	gpio_set_value(gpio_hpe, 1);

	return 0;

err_request_hpe:
err_request_oc_irq:
	gpio_free(gpio_ocd);
err_request_ocd:

	return err;
}

static struct pxaohci_platform_data limestone_ohci_platform_data = {
	.port_mode	= PMM_GLOBAL_MODE,
	.power_budget	= 500,
	.init		= limestone_ohci_init,
	.flags		= ENABLE_PORT2,		/* Enable only OTG port 2. */
};

static void __init limestone_init_usb_ohci(void)
{
	/* set OHCI platform parameters */
	pxa_set_ohci_info(&limestone_ohci_platform_data);
}
#else /* CONFIG_USB_OHCI_HCD || CONFIG_USB_OHCI_HCD_MODULE */
static inline void limestone_init_usb_ohci(void) {}
#endif /* CONFIG_USB_OHCI_HCD || CONFIG_USB_OHCI_HCD_MODULE */

#if defined(CONFIG_LEDS_GPIO) || defined(CONFIG_LEDS_GPIO_MODULE)
/* Vibra motor */
static struct gpio_led limestone_vibra[] = {
	{
		.name = "vibra0::vibra",
		.gpio = LIMESTONE_GPIO_VIBRA,
	},
};

static struct gpio_led_platform_data limestone_vibra_data = {
	.num_leds	= ARRAY_SIZE(limestone_vibra),
	.leds		= limestone_vibra
};

static struct platform_device limestone_vibra_dev = {
	.name		= "leds-gpio",
	.id		= 0,
	.dev		= {
		.platform_data	= &limestone_vibra_data,
	},
};

static void __init limestone_init_vibra(void)
{
	platform_device_register(&limestone_vibra_dev);
}
#else /* CONFIG_LEDS_GPIO || CONFIG_LEDS_GPIO_MODULE */
static inline void limestone_init_vibra(void) {}
#endif /* CONFIG_LEDS_GPIO || CONFIG_LEDS_GPIO_MODULE */

static void __init limestone_init(void)
{
	/* initialize MFP */
	pxa3xx_mfp_config(ARRAY_AND_SIZE(mfp_cfg));

	limestone_init_ac97();
	limestone_init_fb();
	limestone_init_i2c();
	colibri_pxa_init_gpio_keys(PXA_LIMESTONE_CB);
	limestone_init_nand();
	limestone_init_power();
	limestone_init_spi();
	limestone_init_uart();
	limestone_init_udc();
	limestone_init_usb_android();
	limestone_init_usb_ohci();
	limestone_init_vibra();
}

/* Limestone P62/T1 V1.2b */
MACHINE_START(LIMESTONE, "Toradex Limestone PDA Design Kit")
	.boot_params	= 0xa0000100,
	.init_irq	= pxa3xx_init_irq,
	.init_machine	= limestone_init,
	.io_pg_offst	= (io_p2v(0x40000000) >> 18) & 0xfffc,
	.map_io		= pxa_map_io,
	.phys_io	= 0x40000000,
	.timer		= &pxa_timer,
MACHINE_END
